/**
 * Generates some fake data and writes it to a sibling data.json file if the
 * file does not exist. The data is roughly coherent but might contains some
 * integrity issues.
 */

const f = require('@ngneat/falso');
const fs = require('fs');

const OUTPUT_FILENAME = __dirname + '/data.json';

const CHANNELS_5G = [
  7, 8, 9, 11, 12, 16, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 54, 56, 58, 60,
  62, 64, 100, 102, 104, 106, 108, 110, 112, 114, 116, 118, 120, 122, 124, 126,
  128, 132, 134, 136, 138, 140, 142, 144, 149, 151, 153, 155, 157, 159, 161,
  165, 183, 184, 185, 187, 188, 189, 192, 196,
];

const CHANNELS_2G = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14];

const rooms = [
  'Game room',
  'Main working area',
  'Conference room',
  'Kitchen',
  'Hardware prototype',
  'Lobby',
  'Stairs',
  'Entrance',
  'Elevators',
  'Storage',
  'Closet',
  'Micro kitchen',
];

function shortId() {
  return f.randUuid().slice(0, 8);
}

const networks = Array(20)
  .fill(null)
  .map(() => {
    const name = f.randOAuthProvider();
    return {
      id: shortId(),
      created_at: f.randRecentDate(),
      name: name + ' Net',
      customer_name: name,
      hostname: name.replace(/[^\w]+/g, '-').toLocaleLowerCase() + '-net',
      address: `${f.randStreetAddress()}, ${f.randCity()}, ${f.randStateAbbr()} ${f.randZipCode()}`,
      square_feet: f.randNumber({ min: 2000, max: 1000000 }),
      updated_at: f.randRecentDate(),
      status: f.rand(['online', 'offline']),
    };
  });

const access_points = networks
  .map((n, ni) => {
    return Array(f.randNumber({ min: 1, max: 8 }))
      .fill(null)
      .map((d, di) => {
        const id = shortId();
        const status = f.rand(['online', 'offline']);
        const isOnline = status === 'online';
        const location = f.rand(rooms);

        return {
          access_point: {
            id: id,
            version: f.randSemver(),
            location: location,
            created_at: f.randPastDate(),
            updated_at: f.randRecentDate(),
            network_id: n.id,
            name: `AP${ni + 1}.${di + 1}`,
            connected_clients: isOnline ? f.randNumber({ min: 0, max: 20 }) : 0,
            status: status,
          },
          radios: [
            {
              band: 'RB_5G',
              channel: f.rand(CHANNELS_5G),
              channel_width: f.rand([20, 40, 80]),
              power: f.randNumber({ min: 10, max: 20 }),
              disabled: f.randBoolean(),
              created_at: f.randRecentDate(),
              updated_at: f.randRecentDate(),
            },
            {
              band: 'RB_2G',
              channel: f.rand(CHANNELS_2G),
              channel_width: 20,
              power: f.randNumber({ min: 10, max: 20 }),
              disabled: f.randBoolean(),
              created_at: f.randRecentDate(),
              updated_at: f.randRecentDate(),
            },
          ],
        };
      });
  })
  .flat();

function createLease(networkId) {
  return {
    expiry: f.randSoonDate(),
    mac: f.randMac().replace(/-/g, ':'),
    ip: f.randIp({ mask: '10.0.1.1/24' }),
    name:
      f.randFirstName({ withAccents: false }) +
      f.rand(['s-MBP', 's-macbook', '-laptop', '', '-iPhone', 'iPad']),
    network_id: networkId,
  };
}

const wiredClients = networks
  .map((n) =>
    Array(f.randNumber({ min: 0, max: 10 }))
      .fill(null)
      .map(() => {
        const connected = f.randBoolean();
        return {
          id: shortId(),
          connection_type: 'wired',
          lease: createLease(n.id),
          connected: connected,
          last_seen: f.randRecentDate({ days: 1 }),
        };
      }),
  )
  .flat();

const wirelessClients = access_points
  .map((ap) =>
    Array(ap.access_point.connected_clients)
      .fill(null)
      .map(() => {
        const connected = f.randBoolean();

        return {
          id: shortId(),
          connection_type: 'wireless',
          lease: createLease(ap.access_point.network_id),
          wireless_info: {
            access_point_id: ap.access_point.id,
            tx_rate_bytes: f.randNumber({ min: 0, max: 1000000 }),
            rx_rate_bytes: f.randNumber({ min: 0, max: 1000000 }),
            signal_dbm: f.randNumber({ min: -100, max: -4 }),
            noise_dbm: f.randNumber({ min: -100, max: -4 }),
            channel: f.rand([ap.radios[0].channel, ap.radios[1].channel]),
          },
          connected: connected,
          last_seen: f.randRecentDate({ days: 1 }),
        };
      }),
  )
  .flat();

const clients = [...wirelessClients, ...wiredClients];

const stats = networks.map((n) => ({
  network_id: n.id,
  download_bytes: f.randNumber({ min: 0, max: 100000000 }),
  upload_bytes: f.randNumber({ min: 0, max: 100000000 }),
  created_at: f.randRecentDate(),
  bandwidth_updated_at: f.randRecentDate(),
  last_rebooted_at: f.randRecentDate(),
}));

if (require.main === module) {
  if (fs.existsSync(OUTPUT_FILENAME)) {
    return;
  }

  const data = {
    networks,
    clients,
    stats,
    access_points,
  };

  fs.writeFileSync(OUTPUT_FILENAME, JSON.stringify(data, null, 2));
}
