import { Column, useSortBy, useTable } from 'react-table';
import { Box, Table, Tbody, Td, Th, Thead, Tr } from '@chakra-ui/react';
import { TriangleDownIcon, TriangleUpIcon } from '@chakra-ui/icons';
import { Link, LinkProps } from 'react-router-dom';

interface DataTableProps<D extends { href?: string } = {}> {
  columns: Column<D>[];
  data: D[];
  rowIsSelected?: (row: D) => boolean;
  rowLinkProps?: (row: D) => LinkProps | null | undefined;
}

export const DataTable = <D extends object>({
  columns,
  data,
  rowIsSelected,
  rowLinkProps,
}: DataTableProps<D>) => {
  const { getTableProps, getTableBodyProps, headerGroups, rows, prepareRow } =
    useTable({ columns, data }, useSortBy);

  return (
    <Table {...getTableProps()} as="div" sx={{ display: 'table' }}>
      <Thead as="div" sx={{ display: 'table-header-group' }}>
        {headerGroups.map((headerGroup) => (
          <Tr
            {...headerGroup.getHeaderGroupProps()}
            as="div"
            sx={{ display: 'table-row' }}
          >
            {headerGroup.headers.map((column) => (
              <Th
                {...column.getHeaderProps(column.getSortByToggleProps())}
                as="div"
                sx={{ display: 'table-cell' }}
              >
                <Box whiteSpace="nowrap">
                  {column.render('Header')}
                  <Box as="span" pl={2}>
                    {column.isSorted ? (
                      column.isSortedDesc ? (
                        <TriangleDownIcon aria-label="sorted descending" />
                      ) : (
                        <TriangleUpIcon aria-label="sorted ascending" />
                      )
                    ) : (
                      <TriangleUpIcon opacity={0} />
                    )}
                  </Box>
                </Box>
              </Th>
            ))}
          </Tr>
        ))}
      </Thead>
      <Tbody
        {...getTableBodyProps()}
        as="div"
        sx={{ display: 'table-row-group' }}
      >
        {rows.map((row) => {
          prepareRow(row);
          const isSelected = rowIsSelected?.(row.original) ?? false;
          const linkProps = rowLinkProps?.(row.original);

          return (
            <Tr
              {...row.getRowProps()}
              background={isSelected ? 'blue.50' : undefined}
              as={linkProps ? Link : 'div'}
              sx={{
                display: 'table-row',
                _hover: linkProps ? { background: 'gray.50' } : undefined,
              }}
              {...(linkProps as any)}
            >
              {row.cells.map((cell) => (
                <Td
                  {...cell.getCellProps()}
                  as="div"
                  sx={{ display: 'table-cell' }}
                >
                  {cell.render('Cell')}
                </Td>
              ))}
            </Tr>
          );
        })}
      </Tbody>
    </Table>
  );
};
