import { Link, Outlet, useParams } from 'react-router-dom';
import { AccessPointData } from '../types/api';
import { api } from '../api';
import {
  Breadcrumb,
  BreadcrumbItem,
  BreadcrumbLink,
  CloseButton,
  Code,
  Heading,
  Stack,
  Text,
} from '@chakra-ui/react';
import {
  SimpleLabelAndValueContainer,
  SimpleLabelAndValueItem,
} from '../components/SimpleLabelAndValue';
import { OnlineOfflineBadge } from '../components/OnlineOfflineBadge';
import { SimpleCard } from '../components/SimpleCard';
import { Suspense, useMemo } from 'react';
import { Column } from 'react-table';
import { PageLayout } from '../layouts/PageLayout';
import { DataTable } from '../components/DataTable';
import { CenterSpinner } from '../components/CenterSpinner';
import { useQuery } from 'react-query';

export const AccessPointDetailPanel: React.FC = () => {
  const { apId } = useParams<{ apId: string }>();

  const req = useQuery(['ap', apId], () => api.accessPoint.detail(apId!));
  const data = req.data!;

  const radio5G = data.radios.find((r) => r.band === 'RB_5G');
  const radio2G = data.radios.find((r) => r.band === 'RB_2G');

  return (
    <Stack spacing={6}>
      <Stack direction="row" justify="space-between">
        <Heading size="md">{data.access_point.name}</Heading>
        <CloseButton
          size="sm"
          as={Link}
          to={`/network/${data.access_point.network_id}/aps/`}
        />
      </Stack>
      <SimpleLabelAndValueContainer>
        <SimpleLabelAndValueItem
          label="Location"
          value={data.access_point.location}
        />
        <SimpleLabelAndValueItem
          label="Connected Clients"
          value={data.access_point.connected_clients}
        />
        <SimpleLabelAndValueItem
          label="Network"
          value={<Code>{data.access_point.network_id}</Code>}
        />
        <SimpleLabelAndValueItem
          label="Status"
          value={<OnlineOfflineBadge value={data.access_point.status} />}
        />
      </SimpleLabelAndValueContainer>
      <SimpleCard title="5 GHz">
        {radio5G ? (
          <SimpleLabelAndValueContainer>
            <SimpleLabelAndValueItem label="Channel" value={radio5G.channel} />
            <SimpleLabelAndValueItem
              label="Channel Width"
              value={radio5G.channel_width}
            />
            <SimpleLabelAndValueItem label="Power" value={radio5G.power} />
          </SimpleLabelAndValueContainer>
        ) : (
          <Text>5G not configured</Text>
        )}
      </SimpleCard>
      <SimpleCard title="2.4 GHz">
        {radio2G ? (
          <SimpleLabelAndValueContainer>
            <SimpleLabelAndValueItem label="Channel" value={radio2G.channel} />
            <SimpleLabelAndValueItem
              label="Channel Width"
              value={radio2G.channel_width}
            />
            <SimpleLabelAndValueItem label="Power" value={radio2G.power} />
          </SimpleLabelAndValueContainer>
        ) : (
          <Text>2G not configured</Text>
        )}
      </SimpleCard>
    </Stack>
  );
};

export const NetworkAccessPointsPage: React.FC = () => {
  const { networkId, apId } = useParams<{ networkId: string; apId?: string }>();

  const { data } = useQuery(['network', networkId, 'aps'], () =>
    api.network.accessPoints(networkId!),
  );

  const columns = useMemo<Column<AccessPointData>[]>(
    () => [
      {
        Header: 'Name',
        accessor: 'name',
      },
      { Header: 'location', accessor: 'location' },
      {
        Header: 'Clients',
        accessor: 'connected_clients',
      },
      {
        Header: 'status',
        accessor: 'status',
        Cell: (props) => <OnlineOfflineBadge value={props.cell.value} />,
      },
    ],
    [],
  );

  return (
    <PageLayout
      title="Access Points"
      breadcrumbs={
        <Breadcrumb px={6}>
          <BreadcrumbItem>
            <BreadcrumbLink as={Link} to="/">
              Networks
            </BreadcrumbLink>
          </BreadcrumbItem>
          <BreadcrumbItem>
            <BreadcrumbLink as={Link} to={`/network/${networkId!}`}>
              {networkId}
            </BreadcrumbLink>
          </BreadcrumbItem>
          <BreadcrumbItem>
            <BreadcrumbLink
              as={Link}
              to={`/network/${networkId!}/aps/`}
              isCurrentPage
            >
              Access Points
            </BreadcrumbLink>
          </BreadcrumbItem>
        </Breadcrumb>
      }
      content={
        <DataTable
          columns={columns}
          data={data?.map((d) => d.access_point) ?? []}
          rowIsSelected={(row) => row.id === apId}
          rowLinkProps={(row) => ({
            to: `/network/${row.network_id}/aps/${row.id}`,
          })}
        />
      }
      panel={
        apId && (
          <Suspense fallback={<CenterSpinner />}>
            <Outlet />
          </Suspense>
        )
      }
    />
  );
};
