import { Link, Outlet, useParams } from 'react-router-dom';
import { ClientData, WirelessConnectionData } from '../types/api';
import { api } from '../api';
import {
  Breadcrumb,
  BreadcrumbItem,
  BreadcrumbLink,
  CloseButton,
  Heading,
  Stack,
} from '@chakra-ui/react';
import {
  SimpleLabelAndValueContainer,
  SimpleLabelAndValueItem,
} from '../components/SimpleLabelAndValue';
import { OnlineOfflineBadge } from '../components/OnlineOfflineBadge';
import { SimpleCard } from '../components/SimpleCard';
import { Suspense, useMemo } from 'react';
import { Column } from 'react-table';
import { PageLayout } from '../layouts/PageLayout';
import { DataTable } from '../components/DataTable';
import { CenterSpinner } from '../components/CenterSpinner';
import { useQuery } from 'react-query';
import { formatBytesAsBps } from '../helpers/formatBps';
import { formatDbm } from '../helpers/formatDbm';

const WirelessDetails: React.FC<{ wirelessInfo: WirelessConnectionData }> = ({
  wirelessInfo,
}) => {
  const { rx_rate_bytes: rx, tx_rate_bytes: tx } = wirelessInfo;

  return (
    <SimpleLabelAndValueContainer>
      <SimpleLabelAndValueItem
        label="Tx / Rx rate"
        value={`${formatBytesAsBps(tx)} / ${formatBytesAsBps(rx)}`}
      />
      <SimpleLabelAndValueItem label="Channel" value={wirelessInfo.channel} />
      <SimpleLabelAndValueItem
        label="Signal strength"
        value={formatDbm(wirelessInfo.signal_dbm)}
      />
      <SimpleLabelAndValueItem
        label="Noise"
        value={formatDbm(wirelessInfo.noise_dbm)}
      />
    </SimpleLabelAndValueContainer>
  );
};

export const ClientDetailPanel: React.FC = () => {
  const { clientId } = useParams<{ clientId: string }>();

  const req = useQuery(['client', clientId], () =>
    api.client.detail(clientId!),
  );
  const client = req.data!;

  const wirelessDetails =
    client?.connection_type === 'wireless' && client.connected;

  return (
    <Stack spacing={6}>
      <Stack direction="row" justify="space-between">
        <Heading size="md">{client.lease.name}</Heading>
        <CloseButton
          size="sm"
          as={Link}
          to={`/network/${client.lease.network_id}/clients/`}
        />
      </Stack>
      <SimpleLabelAndValueContainer>
        <SimpleLabelAndValueItem label="IP address" value={client.lease.ip} />
        <SimpleLabelAndValueItem label="MAC address" value={client.lease.mac} />
        <SimpleLabelAndValueItem
          label="Connection Type"
          value={client.connection_type}
        />
      </SimpleLabelAndValueContainer>

      {wirelessDetails && (
        <SimpleCard title="Connection Info">
          <WirelessDetails wirelessInfo={client.wireless_info} />
        </SimpleCard>
      )}
    </Stack>
  );
};

export const NetworkClientsPage: React.FC = () => {
  const { networkId, clientId } = useParams<{
    networkId: string;
    clientId?: string;
  }>();

  const { data } = useQuery(['network', networkId, 'clients'], () =>
    api.network.clients(networkId!),
  );

  const columns = useMemo<Column<ClientData>[]>(
    () => [
      {
        Header: 'Name',
        accessor: (client) => client.lease.name,
      },
      {
        Header: 'IP ',
        accessor: (client) => client.lease.ip,
      },
      {
        Header: 'MAC Address',
        accessor: (client) => client.lease.mac,
      },
      {
        Header: 'Connection Type',
        accessor: 'connection_type',
      },
      {
        Header: 'Status',
        accessor: (client) => (client.connected ? 'online' : 'offline'),
        Cell: (props: { cell: { value: 'online' | 'offline' } }) => (
          <OnlineOfflineBadge value={props.cell.value} />
        ),
      },
    ],
    [],
  );

  return (
    <PageLayout
      title="Clients"
      breadcrumbs={
        <Breadcrumb px={6}>
          <BreadcrumbItem>
            <BreadcrumbLink as={Link} to="/">
              Networks
            </BreadcrumbLink>
          </BreadcrumbItem>
          <BreadcrumbItem>
            <BreadcrumbLink as={Link} to={`/network/${networkId!}`}>
              {networkId}
            </BreadcrumbLink>
          </BreadcrumbItem>
          <BreadcrumbItem>
            <BreadcrumbLink
              as={Link}
              to={`/network/${networkId!}/clients/`}
              isCurrentPage
            >
              Clients
            </BreadcrumbLink>
          </BreadcrumbItem>
        </Breadcrumb>
      }
      content={
        <DataTable
          columns={columns}
          data={data ?? []}
          rowIsSelected={(row) => row.id === clientId}
          rowLinkProps={(row) => ({
            to: `/network/${row.lease.network_id}/clients/${row.id}`,
          })}
        />
      }
      panel={
        clientId && (
          <Suspense fallback={<CenterSpinner />}>
            <Outlet />
          </Suspense>
        )
      }
    />
  );
};
