import { format, formatDistanceToNow, parseISO } from 'date-fns';
import {
  Alert,
  Breadcrumb,
  BreadcrumbItem,
  BreadcrumbLink,
  Button,
  chakra,
  Editable,
  EditableInput,
  EditablePreview,
  SimpleGrid,
  Stack,
  Stat,
  StatGroup,
  StatLabel,
  StatNumber,
  Text,
  Tooltip,
} from '@chakra-ui/react';
import { formatBytesAsBps } from '../helpers/formatBps';
import { enUS } from 'date-fns/locale';
import { SimpleCard } from '../components/SimpleCard';
import {
  SimpleLabelAndValueContainer,
  SimpleLabelAndValueItem,
} from '../components/SimpleLabelAndValue';
import { Link, useParams } from 'react-router-dom';
import { PageLayout } from '../layouts/PageLayout';
import { useMutation, useQuery } from 'react-query';
import { api } from '../api';

export const NetworkDetailPage: React.FC = () => {
  const { networkId } = useParams<{ networkId: string }>();

  // TODO: Type mutation correctly
  const networkRename = useMutation(({ name }: { name: string }) =>
    api.network.rename(networkId!, name)
  );

  // TODO: Throw error messages maybe
  const rename = (name: string) => {
    if (!name) return
    if (name.trim().length < 1) return
    networkRename.mutate({ name: name.trim() })
  }

  const summary = useQuery(['network', networkId], () =>
    api.network.detail(networkId!),
  ).data!;

  const stats = useQuery(['network', networkId, 'stats'], () =>
    api.network.stats(networkId!),
  ).data!;

  const aps = useQuery(['network', networkId, 'aps'], () =>
    api.network.accessPoints(networkId!),
  ).data!;

  const onlineApsCount = aps.filter(
    (ap) => ap.access_point.status === 'online',
  ).length;
  const offlineApsCount = aps.filter(
    (ap) => ap.access_point.status === 'offline',
  ).length;

  const clients = useQuery(['network', networkId, 'clients'], () =>
    api.network.clients(networkId!),
  ).data!;

  const wirelessClientsCount = clients.filter(
    (client) => client.connected && client.connection_type === 'wireless',
  ).length;
  const wiredClientsCount = clients.filter(
    (client) => client.connected && client.connection_type === 'wired',
  ).length;
  const offlineClientsCount = clients.filter(
    (client) => !client.connected,
  ).length;

  const bandwidthUpdatedAt = parseISO(stats.bandwidth_updated_at);

  return (
    <PageLayout
      title={
        <>
          {/* TODO: Prevent default editable if value is empty */}
          <Editable defaultValue={summary.name} onSubmit={rename}>
            <EditablePreview />
            <EditableInput />
          </Editable>
          { networkRename.isLoading ? (
            <Text>{`Updating name to ${name}...`}</Text>
          ) : (
            <>
              {networkRename.isError && (
                <Alert status='error'> {networkRename?.error?.message} </Alert>
              )}
              {networkRename.isSuccess && (<Alert status='success'> Network renamed! </Alert>)}
            </>
          )}
        </>
      }
      breadcrumbs={
        <Breadcrumb px={6}>
          <BreadcrumbItem>
            <BreadcrumbLink as={Link} to="/">
              Networks
            </BreadcrumbLink>
          </BreadcrumbItem>
          <BreadcrumbItem>
            <BreadcrumbLink
              as={Link}
              to={`/network/${networkId}`}
              isCurrentPage
            >
              {networkId}
            </BreadcrumbLink>
          </BreadcrumbItem>
        </Breadcrumb>
      }
      content={
        <Stack spacing={6} px={6}>
          <SimpleGrid columns={2} gridColumnGap={6}>
            <SimpleLabelAndValueContainer>
              <SimpleLabelAndValueItem
                label="Customer"
                value={summary.customer_name}
              />
              <SimpleLabelAndValueItem
                label="Address"
                value={summary.address}
              />
              <SimpleLabelAndValueItem
                label="Square footage"
                value={summary.square_feet}
              />
            </SimpleLabelAndValueContainer>
            <Stack spacing={0}>
              <StatGroup>
                <Stat>
                  <StatLabel>Download Speed</StatLabel>
                  <StatNumber>
                    {formatBytesAsBps(stats.download_bytes)}
                  </StatNumber>
                </Stat>
                <Stat>
                  <StatLabel>Upload Speed</StatLabel>
                  <StatNumber>
                    {formatBytesAsBps(stats.upload_bytes)}
                  </StatNumber>
                </Stat>
              </StatGroup>
              <Text color="gray.500">
                Last updated{' '}
                <Tooltip
                  label={format(bandwidthUpdatedAt, 'PPPppp', { locale: enUS })}
                >
                  <chakra.span borderBottom="1px dashed" borderColor="gray.300">
                    {formatDistanceToNow(bandwidthUpdatedAt, { locale: enUS })}{' '}
                    ago
                  </chakra.span>
                </Tooltip>
              </Text>
            </Stack>
          </SimpleGrid>
          <SimpleGrid columns={2} gridColumnGap={6}>
            <SimpleCard
              title="Access Points"
              action={
                <Button size="sm" as={Link} to={`/network/${networkId!}/aps/`}>
                  View all
                </Button>
              }
            >
              <StatGroup>
                <Stat>
                  <StatLabel>Online</StatLabel>
                  <StatNumber color="green.600">{onlineApsCount}</StatNumber>
                </Stat>
                <Stat>
                  <StatLabel>Offline</StatLabel>
                  <StatNumber color="red.600">{offlineApsCount}</StatNumber>
                </Stat>
              </StatGroup>
            </SimpleCard>
            <SimpleCard
              title="Clients"
              action={
                <Button
                  size="sm"
                  as={Link}
                  to={`/network/${networkId!}/clients/`}
                >
                  View all
                </Button>
              }
            >
              <StatGroup>
                <Stat>
                  <StatLabel>Wireless</StatLabel>
                  <StatNumber color="green.600">
                    {wirelessClientsCount}
                  </StatNumber>
                </Stat>
                <Stat>
                  <StatLabel>Wired</StatLabel>
                  <StatNumber color="green.600">{wiredClientsCount}</StatNumber>
                </Stat>
                <Stat>
                  <StatLabel>Offline</StatLabel>
                  <StatNumber color="red.600">{offlineClientsCount}</StatNumber>
                </Stat>
              </StatGroup>
            </SimpleCard>
          </SimpleGrid>
        </Stack>
      }
    />
  );
};
